from linkauto.api.dataclasses_ import Device, LinkedinApp

GooglePixelUS = Device(
  osName='Android OS',
  osVersion='25',
  androidVersion='Android 7.1.1',
  region='en_US',
  cronetVersion='83.0.4103.83',
  model='unknown_Google Pixel XL',
  displayDensity=3.5,
  displayWidth=1440,
  displayHeight=2392,
  dpi='xhdpi',
  deviceType='android',
  timezoneOffset=-4,
  timezone='America/New_York',
  isAdTrackingLimited=False,
)

Linkedin_41588_146600 = LinkedinApp(
  clientVersion='4.1.558',
  clientMinorVersion=146600,
  appId='com.linkedin.android',
  storeId='us_googleplay',
  mpName='voyager-android',
  mpVersion='0.6.34.153'
)
